﻿Imports System.Text

Public Class CaptureWindow

    Public ulPrivateNumber As ULong

    Private Sub btnCancel_Click(sender As System.Object, e As System.EventArgs) Handles btnCancel.Click
        ' キャンセル
        Me.Close()
    End Sub

    Private Sub btnSave_Click(sender As System.Object, e As System.EventArgs) Handles btnSave.Click
        ' 保存

        Dim strExt As String
        Dim strFileName As String = txtFileName.Text

        Try
            ' 拡張子部分の取り出し
            strExt = System.IO.Path.GetExtension(txtFileName.Text).ToLower

            ' ■フォルダの存在確認
            ' フォルダ名部分の取り出し
            Dim strDirName As String = System.IO.Path.GetDirectoryName(strFileName)

            ' ファイル名形式のチェック(この時点で、* と ? はチェックをすり抜けているので、個別にチェックする)
            ' (ちなみに、不正な位置に : がある場合は、残念ながらすり抜けることになる)
            If (strDirName.IndexOf("*"c) <> -1 OrElse strDirName.IndexOf("?"c) <> -1) Then
                MessageBox.Show("ファイル名の指定が不正です。ファイル名を見直してください。", "ファイルへ保存", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Return
            End If

            If Not System.IO.Directory.Exists(strDirName) Then
                If DialogResult.No = MessageBox.Show("フォルダ " + strDirName + " は存在しません。フォルダを新規作成しますか?。", "ファイルへ保存", MessageBoxButtons.YesNo, MessageBoxIcon.Question) Then
                    Return
                End If

                Try
                    System.IO.Directory.CreateDirectory(strDirName)
                Catch ex As Exception
                    ErrorDialog.ShowMessage("新規フォルダを作成できません。フォルダ名を見直してください。", ex.ToString(), "ファイルへ保存")
                    Return
                End Try
            End If
        Catch ex As Exception
            ErrorDialog.ShowMessage("ファイル名の指定が不正です。ファイル名を見直してください。", ex.ToString(), "ファイルへ保存")
            Return
        End Try

        Try
            Dim formatkind As System.Drawing.Imaging.ImageFormat

            If MainWindow.intFormat = 0 Then
                ' Jpegで保存
                If Not (strExt = ".jpg" OrElse strExt = ".jpeg") Then
                    strFileName += ".jpg"
                End If

                formatkind = System.Drawing.Imaging.ImageFormat.Jpeg
            Else
                ' Pngで保存
                If Not strExt = ".png" Then
                    strFileName += ".png"
                End If

                formatkind = System.Drawing.Imaging.ImageFormat.Png
            End If

            If System.IO.File.Exists(strFileName) Then
                If DialogResult.Cancel = MessageBox.Show(strFileName + "はすでに存在します。上書きしますか?", "ファイルへ保存", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) Then
                    Return
                End If
            End If

            PictureBox1.Image.Save(strFileName, formatkind)

            btnCancel.Visible = False
            btnSave.Visible = False
            txtFileName.Enabled = False

            lblInfo.Visible = False
            lblInfo2.Visible = True

            ' 1500ミリ秒後にクローズ
            Timer_End.Start()

        Catch ex As System.NotSupportedException
            ErrorDialog.ShowMessage(strFileName + "へ保存できません。ファイル名の形式が不正であるため、ファイル名を見直してください。", ex.ToString(), "ファイルへ保存")

        Catch ex As System.ArgumentException
            ErrorDialog.ShowMessage(strFileName + "へ保存できません。ファイル名に不正な文字が含まれているため、ファイル名を見直してください。", ex.ToString(), "ファイルへ保存")

        Catch ex As Exception
            ErrorDialog.ShowMessage(strFileName + "へ保存できません。", ex.ToString(), "ファイルへ保存")
        End Try
    End Sub

    Private Sub CaptureWindow_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load
        Dim dtNow As Date = Now()
        Dim intSelectCount As Integer = 0

        ' フォームが開かれたら
        ' クリップボードから画像を取り出す
        ' (クリップボードの中身が画像でなければフォームを閉じる)
        Try
            If Clipboard.ContainsImage() Then
                'クリップボードにあるデータの取得
                Dim img As Image = Clipboard.GetImage()
                If img IsNot Nothing Then
                    'データが取得できたときは表示する
                    PictureBox1.Image = img
                End If
            End If

        Catch ex As Exception
            btnCancel.Visible = False
            btnSave.Visible = False
            txtFileName.Enabled = False

            ErrorDialog.ShowMessage("クリップボードからのデータ取得に失敗しました。", ex.ToString(), "ファイルへ保存")

            ' 1500ミリ秒後にクローズ
            Timer_End.Start()
        End Try

        ' ファイル名の組み立て
        Dim stFileName As StringBuilder

        If MainWindow.bolShosai = False Then
            ' [簡易指定の場合]
            ' ベースファイル名の取得
            stFileName = New StringBuilder(MainWindow.strFileName)

            ' 日付時刻
            If MainWindow.bAddDateTime Then
                stFileName.Append(dtNow.ToString("yyyyMMdd-HHmmss-"))
                intSelectCount += 16
            End If

            ' 通番部分
            If MainWindow.bAddNumber Then
                stFileName.Append(MainWindow.intNumber.ToString("D8"))
                MainWindow.intNumber += 1 ' 次回通番を+1
                If (MainWindow.intNumber > 99999999) Then
                    ' 99999999を超えたら0にする(初期値は1だが、この場合だけ0で)
                    MainWindow.intNumber = 0
                End If

                intSelectCount += 8
            End If
        Else
            ' [詳細指定の場合]
            stFileName = New StringBuilder(MainWindow.strFileName2)

            ' ファイル名のフォーマットを1つずつ変換
            '<%t4y> : 年(4桁)
            stFileName.Replace("<%t4y>", dtNow.ToString("yyyy"))
            '<%t2y> : 年(2桁)
            stFileName.Replace("<%t2y>", dtNow.ToString("yy"))
            '<%t0m> : 月(2桁)(1桁のときは、10の位は0)
            stFileName.Replace("<%t0m>", dtNow.ToString("MM"))
            '<%tm>  : 月(1桁)(1桁のときは、10の位は無し)
            stFileName.Replace("<%tm>", dtNow.ToString("%M"))
            '<%tem> : 月(英語表記)
            stFileName.Replace("<%tem>", dtNow.ToString("MMMM"))
            '<%tsem>: 月(英語表記省略形)
            stFileName.Replace("<%tsem>", dtNow.ToString("MMM"))
            '<%t0d> : 日(2桁)(1桁のときは、10の位は0)
            stFileName.Replace("<%t0d>", dtNow.ToString("dd"))
            '<%td>  : 日(1桁)(1桁のときは、10の位は無し)
            stFileName.Replace("<%td>", dtNow.ToString("%d"))
            '<%t0h> : 時(2桁)(1桁のときは、10の位は0)
            stFileName.Replace("<%t0h>", dtNow.ToString("HH"))
            '<%th>  : 時(1桁)(1桁のときは、10の位は無し)
            stFileName.Replace("<%th>", dtNow.ToString("%H"))
            '<%t0n> : 分(2桁)(1桁のときは、10の位は0)
            stFileName.Replace("<%t0n>", dtNow.ToString("mm"))
            '<%tn>  : 分(1桁)(1桁のときは、10の位は無し)
            stFileName.Replace("<%tn>", dtNow.ToString("%m"))
            '<%t0s> : 秒(2桁)(1桁のときは、10の位は0)
            stFileName.Replace("<%t0s>", dtNow.ToString("ss"))
            '<%ts>  : 秒(1桁)(1桁のときは、10の位は無し)
            stFileName.Replace("<%ts>", dtNow.ToString("%s"))
            '<%tms> : ミリ秒
            stFileName.Replace("<%tms>", dtNow.ToString("fff"))
            '<%n?>   : 連番(最低?桁)
            stFileName.Replace("<%n8>", MainWindow.intNumber.ToString("D8"))
            stFileName.Replace("<%n7>", MainWindow.intNumber.ToString("D7"))
            stFileName.Replace("<%n6>", MainWindow.intNumber.ToString("D6"))
            stFileName.Replace("<%n5>", MainWindow.intNumber.ToString("D5"))
            stFileName.Replace("<%n4>", MainWindow.intNumber.ToString("D4"))
            stFileName.Replace("<%n3>", MainWindow.intNumber.ToString("D3"))
            stFileName.Replace("<%n2>", MainWindow.intNumber.ToString("D2"))
            stFileName.Replace("<%n1>", MainWindow.intNumber.ToString("D1"))

            MainWindow.intNumber += 1 ' 次回通番を+1
            If (MainWindow.intNumber > 99999999) Then
                ' 99999999を超えたら0にする(初期値は1だが、この場合だけ0で)
                MainWindow.intNumber = 0
            End If
        End If

        ' 拡張子を付与
        If MainWindow.intFormat = 0 Then
            ' Jpegで保存
            stFileName.Append(".jpg")
        Else
            ' Pngで保存
            stFileName.Append(".png")
        End If

        ' ファイル名の完成
        txtFileName.Text = stFileName.ToString

        If MainWindow.bolShosai = False Then
            ' [簡易指定の場合]
            ' 通番部分・日付部分がある場合は、通番を選択状態にする
            ' 通番部分・日付部分がない場合は、通番・日付があるはずの位置にカーソルを合わせる
            txtFileName.Select(MainWindow.strFileName.Length, intSelectCount)
        Else
            ' [詳細指定の場合]
            '*  : カーソル位置(始めの1つのみが有効)
            Dim intSelectedStartPos As Integer

            intSelectedStartPos = txtFileName.Text.IndexOf("*"c)
            If intSelectedStartPos <> -1 Then
                txtFileName.Text = txtFileName.Text.Replace("*", "")
                txtFileName.Select(intSelectedStartPos, 0)
            Else
                txtFileName.Select(txtFileName.Text.Length - 4, 0)
            End If
        End If

        ' 現在時刻の表示
        lblInfo.Text = dtNow.ToString()

        ' プライマリ画面を取得し、自身が表示されるべき場所を取得
        Me.Top = Screen.GetWorkingArea(Me).Height - Me.Height
        Me.Left = Screen.GetWorkingArea(Me).Width - Me.Width
    End Sub

    Private Sub CaptureWindow_FormClosing(sender As System.Object, e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing
        ' クローズする際はフォーム１にプールからの削除を依頼する
        MainWindow.DelCaptureWindow(ulPrivateNumber)
    End Sub

    Private Sub Timer1_Tick(sender As System.Object, e As System.EventArgs) Handles Timer_End.Tick, lblInfo2.Click
        Timer_End.Stop()
        Me.Close()
    End Sub

    Private Sub Timer_Start_Tick(sender As System.Object, e As System.EventArgs) Handles Timer_Start.Tick
        ' 必ず最前面に表示されつつも、以降は最前面にならないようにするため
        Timer_Start.Stop()
        Me.TopMost = True
        Me.Refresh()
        Me.TopMost = False
    End Sub
End Class